@@ -22,6 +22,7 @@
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 
 #include "packet.h"
@@ -35,11 +36,13 @@
 #include "cipher.h"
 #include "filter.h"
 #include "trustdb.h"
+#include "ttyio.h"
 
 enum cmd_values { aNull = 0,
     aSym, aStore, aEncr, aPrimegen, aKeygen, aSign, aSignEncr,
     aPrintMDs, aSignKey, aClearsig, aListPackets, aEditSig,
-    aKMode, aKModeC, aChangePass,
+    aKMode, aKModeC, aChangePass, aImport, aListTrustDB,
+    aListTrustPath,
 aTest };
 
 
@@ -88,6 +91,15 @@ strusage( int level )
     return p;
 }
 
+static void
+wrong_args( const char *text)
+{
+    fputs("Usage: g10 [options] ",stderr);
+    fputs(text,stderr);
+    putc('\n',stderr);
+    exit(2);
+}
+
 static void
 set_debug(void)
 {
@@ -174,11 +186,18 @@ main( int argc, char **argv )
     { 527, "cipher-algo", 2 , "select default cipher algorithm" },
     { 528, "pubkey-algo", 2 , "select default puplic key algorithm" },
     { 529, "digest-algo", 2 , "select default message digest algorithm" },
+    { 530, "import",      0 , "put public keys into the trustdb" },
+    { 531, "list-trustdb",0 , "\r"},
+    { 532, "quick-random", 0, "\r"},
+    { 533, "list-trust-path",0, "\r"},
+    { 534, "no-comment", 0,   "do not write comment packets"},
+    { 535, "completes_needed", 1, "(default is 1)"},
+    { 536, "marginals_needed", 1, "(default is 3)"},
 
     {0} };
     ARGPARSE_ARGS pargs;
     IOBUF a;
-    int rc;
+    int rc=0;
     int orig_argc;
     char **orig_argv;
     const char *fname, *fname_print;
@@ -202,6 +221,8 @@ main( int argc, char **argv )
     opt.def_cipher_algo = CIPHER_ALGO_BLOWFISH;
     opt.def_pubkey_algo = PUBKEY_ALGO_ELGAMAL;
     opt.def_digest_algo = DIGEST_ALGO_RMD160;
+    opt.completes_needed = 1;
+    opt.marginals_needed = 3;
 
     /* check wether we have a config file on the commandline */
     orig_argc = argc;
@@ -316,9 +337,13 @@ main( int argc, char **argv )
 	  case 529:
 	    opt.def_digest_algo = string_to_digest_algo(pargs.r.ret_str);
 	    break;
-
-
-	    break;
+	  case 530: set_cmd( &cmd, aImport); break;
+	  case 531: set_cmd( &cmd, aListTrustDB); break;
+	  case 532: quick_random_gen(1); break;
+	  case 533: set_cmd( &cmd, aListTrustPath); break;
+	  case 534: opt.no_comment=1; break;
+	  case 535: opt.completes_needed = pargs.r.ret_int; break;
+	  case 536: opt.marginals_needed = pargs.r.ret_int; break;
 	  default : errors++; pargs.err = configfp? 1:2; break;
 	}
     }
@@ -341,9 +366,18 @@ main( int argc, char **argv )
 	log_error("selected digest algorithm is invalid\n");
 	errors++;
     }
+    if( opt.completes_needed < 1 ) {
+	log_error("completes_needed must be greater than 0\n");
+	errors++;
+    }
+    if( opt.marginals_needed < 2 ) {
+	log_error("marginals_needed must be greater than 1\n");
+	errors++;
+    }
     if( errors )
 	exit(2);
 
+
     set_debug();
     if( cmd == aKMode || cmd == aKModeC ) { /* kludge to be compatible to pgp */
 	if( cmd == aKModeC ) {
@@ -393,11 +427,15 @@ main( int argc, char **argv )
 	}
     }
 
-    if( cmd != aPrimegen && cmd != aPrintMDs ) {
-	rc = check_trustdb(0);
-	if( rc )
-	    log_error("failed to initialize the TrustDB: %s\n", g10_errstr(rc));
+    switch( cmd ) {
+      case aPrimegen:
+      case aPrintMDs:
+	break;
+      case aListTrustDB: rc = init_trustdb( argc? 1:0 ); break;
+      default: rc = init_trustdb(1); break;
     }
+    if( rc )
+	log_error("failed to initialize the TrustDB: %s\n", g10_errstr(rc));
 
 
     switch( cmd ) {
@@ -424,17 +462,36 @@ main( int argc, char **argv )
 	break;
 
       case aSign: /* sign the given file */
-	if( argc > 1 )
-	    usage(1);
-	if( (rc = sign_file(fname, detached_sig, locusr, 0, NULL)) )
-	    log_error("sign_file('%s'): %s\n", fname_print, g10_errstr(rc) );
+	sl = NULL;
+	if( detached_sig ) { /* sign all files */
+	    for( ; argc; argc--, argv++ )
+		add_to_strlist( &sl, *argv );
+	}
+	else {
+	    if( argc > 1 )
+		usage(1);
+	    if( argc ) {
+		sl = m_alloc_clear( sizeof *sl + strlen(fname));
+		strcpy(sl->d, fname);
+	    }
+	}
+	if( (rc = sign_file( sl, detached_sig, locusr, 0, NULL, NULL)) )
+	    log_error("sign_file: %s\n", g10_errstr(rc) );
+	free_strlist(sl);
 	break;
 
       case aSignEncr: /* sign and encrypt the given file */
 	if( argc > 1 )
 	    usage(1);
-	if( (rc = sign_file(fname, detached_sig, locusr, 1, remusr)) )
+	if( argc ) {
+	    sl = m_alloc_clear( sizeof *sl + strlen(fname));
+	    strcpy(sl->d, fname);
+	}
+	else
+	    sl = NULL;
+	if( (rc = sign_file(sl, detached_sig, locusr, 1, remusr, NULL)) )
 	    log_error("sign_file('%s'): %s\n", fname_print, g10_errstr(rc) );
+	free_strlist(sl);
 	break;
 
 
@@ -468,7 +525,7 @@ main( int argc, char **argv )
 	    int i, seq=0;
 	    const char *s;
 
-	    while( s=get_keyring(seq++) ) {
+	    while( (s=get_keyring(seq++)) ) {
 		if( !(a = iobuf_open(s)) ) {
 		    log_error("can't open '%s'\n", s);
 		    continue;
@@ -535,6 +592,32 @@ main( int argc, char **argv )
 
       case aTest: do_test( argc? atoi(*argv): 0 ); break;
 
+      case aImport:
+	if( !argc  )
+	    usage(1);
+	for( ; argc; argc--, argv++ ) {
+	    rc = import_pubkeys( *argv );
+	    if( rc )
+		log_error("import from '%s' failed: %s\n",
+						*argv, g10_errstr(rc) );
+	}
+	break;
+
+      case aListTrustDB:
+	if( !argc )
+	    list_trustdb(NULL);
+	else {
+	    for( ; argc; argc--, argv++ )
+		list_trustdb( *argv );
+	}
+	break;
+
+      case aListTrustPath:
+	if( argc != 2 )
+	    wrong_args("--list-trust-path  <maxdepth> <username>");
+	list_trust_path( atoi(*argv), argv[1] );
+	break;
+
       case aListPackets:
 	opt.list_packets=1;
       default:
@@ -612,8 +695,6 @@ print_mds( const char *fname )
     if( ferror(fp) )
 	log_error("%s: %s\n", fname, strerror(errno) );
     else {
-	byte *p;
-
 	md_final(md);
 	printf(  "%s:    MD5 =", fname ); print_hex(md_read(md, DIGEST_ALGO_MD5), 16 );
 	printf("\n%s: RMD160 =", fname ); print_hex(md_read(md, DIGEST_ALGO_RMD160), 20 );
