@@ -41,14 +41,14 @@ int encrypt_filter( void *opaque, int control,
 
 
 /*-- sign.c --*/
-int sign_file( const char *filename, int detached, STRLIST locusr,
-	       int encrypt, STRLIST remusr );
+int sign_file( STRLIST filenames, int detached, STRLIST locusr,
+	       int encrypt, STRLIST remusr, const char *outfile );
 int sign_key( const char *username, STRLIST locusr );
 int edit_keysigs( const char *username );
 int change_passphrase( const char *username );
 
 /*-- sig-check.c --*/
-int check_key_signature( KBNODE root, KBNODE node );
+int check_key_signature( KBNODE root, KBNODE node, int *is_selfsig );
 
 /*-- keygen.c --*/
 void generate_keypair(void);
@@ -77,5 +77,8 @@ void g10_elg_sign( PKT_secret_cert *skc, PKT_signature *sig, MD_HANDLE md );
 void g10_rsa_encrypt( PKT_public_cert *pkc, PKT_pubkey_enc *enc, DEK *dek );
 void g10_rsa_sign( PKT_secret_cert *skc, PKT_signature *sig, MD_HANDLE md );
 
+/*-- import.c --*/
+int import_pubkeys( const char *filename );
+
 
 #endif /*G10_MAIN_H*/
