@@ -58,20 +58,38 @@ typedef struct {
 /*-- logger.c --*/
 void log_set_pid( int pid );
 int  log_get_errorcount( int clear );
-void printstr( int level, const char *fmt, ... );
-void log_bug( const char *fmt, ... );
-void log_fatal( const char *fmt, ... );
-void log_error( const char *fmt, ... );
-void log_info( const char *fmt, ... );
-void log_debug( const char *fmt, ... );
 void log_hexdump( const char *text, char *buf, size_t len );
 void log_mpidump( const char *text, MPI a );
 
+#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 5 )
+  void printstr( int level, const char *fmt, ... )
+			    __attribute__ ((format (printf,2,3)));
+  void log_bug( const char *fmt, ... )
+			    __attribute__ ((noreturn, format (printf,1,2)));
+  void log_bug0( void ) __attribute__ ((noreturn));
+  void log_fatal( const char *fmt, ... )
+			    __attribute__ ((noreturn, format (printf,1,2)));
+  void log_error( const char *fmt, ... ) __attribute__ ((format (printf,1,2)));
+  void log_info( const char *fmt, ... )  __attribute__ ((format (printf,1,2)));
+  void log_debug( const char *fmt, ... ) __attribute__ ((format (printf,1,2)));
+#else
+  void printstr( int level, const char *fmt, ... );
+  void log_bug( const char *fmt, ... );
+  void log_bug0( void );
+  void log_fatal( const char *fmt, ... );
+  void log_error( const char *fmt, ... );
+  void log_info( const char *fmt, ... );
+  void log_debug( const char *fmt, ... );
+#endif
+
+
 /*-- errors.c --*/
 const char * g10_errstr( int no );
 
 /*-- argparse.c --*/
 int arg_parse( ARGPARSE_ARGS *arg, ARGPARSE_OPTS *opts);
+int optfile_parse( FILE *fp, const char *filename, unsigned *lineno,
+		   ARGPARSE_ARGS *arg, ARGPARSE_OPTS *opts);
 void usage( int level );
 const char *default_strusage( int level );
 
@@ -91,6 +109,7 @@ int answer_is_yes( const char *s );
 /*-- strgutil.c --*/
 void free_strlist( STRLIST sl );
 #define FREE_STRLIST(a) do { free_strlist((a)); (a) = NULL ; } while(0)
+void add_to_strlist( STRLIST *list, const char *string );
 char *memistr( char *buf, size_t buflen, const char *sub );
 char *trim_spaces( char *string );
 int string_count_chr( const char *string, int c );
@@ -102,7 +121,9 @@ char *stpcpy(char *a,const char *b);
 #ifndef HAVE_STRLWR
 char *strlwr(char *a);
 #endif
-
+#ifndef HAVE_STRTOUL
+  #define strtoul(a,b,c)  ((unsigned long)strtol((a),(b),(c)))
+#endif
 
 /******** some macros ************/
 #ifndef STR
@@ -111,5 +132,6 @@ char *strlwr(char *a);
 #define STR2(v) STR(v)
 #define DIM(v) (sizeof(v)/sizeof((v)[0]))
 #define DIMof(type,member)   DIM(((type *)0)->member)
+#define BUG() log_bug0()
 
 #endif /*G10_UTIL_H*/
